@@ -258,8 +258,23 @@ class TestRouter(base.RequestsMockTestCase):
         self.assertTrue(self.cloud.delete_router("123"))
         self.assert_calls()
 
+    def _get_mock_dict(self, owner, json):
+        return dict(method='GET',
+                    uri=self.get_mock_url(
+                        'network', 'public', append=['v2.0', 'ports.json'],
+                        qs_elements=["device_id=%s" % self.router_id,
+                                     "device_owner=network:%s" % owner]),
+                    json=json)
+
     def _test_list_router_interfaces(self, router, interface_type,
+                                     router_type="normal",
                                      expected_result=None):
+        if router_type == "normal":
+            device_owner = 'router_interface'
+        elif router_type == "ha":
+            device_owner = 'ha_router_replicated_interface'
+        elif router_type == "dvr":
+            device_owner = 'router_interface_distributed'
         internal_port = {
             'id': 'internal_port_id',
             'fixed_ips': [{
@@ -267,7 +282,7 @@ class TestRouter(base.RequestsMockTestCase):
                 'ip_address': "10.0.0.1"
             }],
             'device_id': self.router_id,
-            'device_owner': 'network:router_interface'
+            'device_owner': 'network:%s' % device_owner
         }
         external_port = {
             'id': 'external_port_id',
@@ -286,56 +301,47 @@ class TestRouter(base.RequestsMockTestCase):
             else:
                 expected_result = [internal_port, external_port]
 
-        self.register_uris([
-            dict(method='GET',
-                 uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'ports.json'],
-                     qs_elements=["device_id=%s" % self.router_id,
-                                  "device_owner=network:router_interface"]),
-                 json={'ports': [internal_port]}),
-            dict(method='GET',
-                 uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'ports.json'],
-                     qs_elements=["device_id=%s" % self.router_id,
-                                  "device_owner=network:router_gateway"]),
-                 json={'ports': [external_port]})
-        ])
+        mock_uris = []
+        for port_type in ['router_interface',
+                          'router_interface_distributed',
+                          'ha_router_replicated_interface']:
+            ports = {}
+            if port_type == device_owner:
+                ports = {'ports': [internal_port]}
+            mock_uris.append(self._get_mock_dict(port_type, ports))
+        mock_uris.append(self._get_mock_dict('router_gateway',
+                                             {'ports': [external_port]}))
+
+        self.register_uris(mock_uris)
         ret = self.cloud.list_router_interfaces(router, interface_type)
         self.assertEqual(expected_result, ret)
         self.assert_calls()
 
-    def test_list_router_interfaces_all(self):
-        router = {
-            'id': self.router_id,
-            'external_gateway_info': {
-                'external_fixed_ips': [{
-                    'subnet_id': 'external_subnet_id',
-                    'ip_address': '1.2.3.4'}]
-            }
+    router = {
+        'id': router_id,
+        'external_gateway_info': {
+            'external_fixed_ips': [{
+                'subnet_id': 'external_subnet_id',
+                'ip_address': '1.2.3.4'}]
         }
-        self._test_list_router_interfaces(router,
+    }
+
+    def test_list_router_interfaces_all(self):
+        self._test_list_router_interfaces(self.router,
                                           interface_type=None)
 
     def test_list_router_interfaces_internal(self):
-        router = {
-            'id': self.router_id,
-            'external_gateway_info': {
-                'external_fixed_ips': [{
-                    'subnet_id': 'external_subnet_id',
-                    'ip_address': '1.2.3.4'}]
-            }
-        }
-        self._test_list_router_interfaces(router,
+        self._test_list_router_interfaces(self.router,
                                           interface_type="internal")
 
     def test_list_router_interfaces_external(self):
-        router = {
-            'id': self.router_id,
-            'external_gateway_info': {
-                'external_fixed_ips': [{
-                    'subnet_id': 'external_subnet_id',
-                    'ip_address': '1.2.3.4'}]
-            }
-        }
-        self._test_list_router_interfaces(router,
+        self._test_list_router_interfaces(self.router,
                                           interface_type="external")
+
+    def test_list_router_interfaces_internal_ha(self):
+        self._test_list_router_interfaces(self.router, router_type="ha",
+                                          interface_type="internal")
+
+    def test_list_router_interfaces_internal_dvr(self):
+        self._test_list_router_interfaces(self.router, router_type="dvr",
+                                          interface_type="internal")
